@@ -17,19 +17,24 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: error.h 121 2008-01-10 00:17:40Z olivier $
+ *   SVN:$Id: error.h 528 2009-04-19 20:32:47Z olivier $
  */
 
 #ifndef JACKBEAT_ERR_H
 #define JACKBEAT_ERR_H
 
-
-#define ERR_JAB_XML                 1
-#define ERR_SEQUENCE_INTERNAL       2
-#define ERR_SEQUENCE_MSQ            3 // obsolete
-#define ERR_SEQUENCE_JACK_CONNECT   4
-#define ERR_SEQUENCE_JACK_ACTIVATE  5
-#define ERR_SEQUENCE_JACK_REGPORT   6
+enum {
+  ERR_JAB_XML = 1,
+  ERR_JAB_VERSION,
+  ERR_JAB_EXTRACT,
+  ERR_SEQUENCE_INTERNAL,
+  ERR_SEQUENCE_JACK_CONNECT,  
+  ERR_SEQUENCE_JACK_ACTIVATE, 
+  ERR_SEQUENCE_JACK_REGPORT,
+  ERR_SEQUENCE_DUPLICATE_TRACK_NAME,
+  ERR_SEQUENCE_INVALID_NAME,
+  ERR_INTERNAL
+};  
 
 char * error_to_string (int code);
 
