@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: file.c 655 2009-11-26 11:10:55Z olivier $
+ *   SVN:$Id: file.c 661 2009-12-16 12:35:51Z olivier $
  */
 
 #ifdef __WIN32__
@@ -290,7 +290,7 @@ gui_file_export_sequence (gui_t * gui, guint action, GtkWidget * w)
                                               "WAV files", exts); 
     if (chosen_name) {
 
-        int framerate, sustain_type;
+        int framerate, sustain_type = 0;
 
         if (gui_file_get_export_settings (gui, &framerate, &sustain_type)) {
             DEBUG("Chosen name: %s", chosen_name);
@@ -667,11 +667,13 @@ gui_file_dialog_save(const char *title, GtkWidget *parent, const char *filename,
         NULL
     );  
     if (filename) {
-        tmp_filename = strdup (filename);
-        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), dirname(tmp_filename));
-        free (tmp_filename);
-        gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
-                                           basename(filename));
+        tmp_filename = strdup(filename);
+        gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), dirname(tmp_filename));
+        free(tmp_filename);
+        tmp_filename = strdup(filename);
+        gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog),
+                                          basename(tmp_filename));
+        free(tmp_filename);
     }
     gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
     if (filter_name) {
