@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: jab.c 611 2009-06-26 12:53:25Z olivier $
+ *   SVN:$Id: jab.c 640 2009-11-23 19:08:34Z olivier $
  */
 
 #include <stdio.h>
@@ -289,7 +289,7 @@ jab_retrieve_sequence (jab_t *jab, stream_t *stream, char *sequence_name, int *e
       {
        _jab_xml_scan (jab->xml_des, track, "/name", str, JAB_XML_STRING);
        sequence_normalize_name (str);
-       if (sequence_set_track_name (sequence, i, str)) {
+       if (sequence_force_track_name (sequence, i, str)) {
           *error = ERR_INTERNAL;
           sequence_destroy (sequence);
           return NULL;
