@@ -17,9 +17,10 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: main.c 607 2009-06-25 15:49:51Z olivier $
+ *   SVN:$Id: main.c 682 2010-09-08 14:28:19Z olivier $
  */
 
+#define _GNU_SOURCE // For REG_RIP from ucontext.h
 
 #include <string.h>
 #include <config.h>
@@ -61,44 +62,68 @@
 #include <execinfo.h>
 
 /* get REG_EIP from ucontext.h */
-#define __USE_GNU
 #include <ucontext.h>
 
-void bt_sighandler(int sig, siginfo_t *info,
-				   void *secret) {
+void print_backtrace(void **trace, int size) {
+    printf("CAUGHT SIGSEGV >> DUMPING BACKTRACE\n");
 
-  printf ("CAUGHT SIGSEV >> DUMPING BACKTRACE\n");
+    backtrace_symbols_fd(trace + 1, size - 1, STDOUT_FILENO);
 
-  void *trace[200];
-  int trace_size = 0;
-  ucontext_t *uc = (ucontext_t *)secret;
+    printf("#####################################################################\n");
+    printf("#                             KRASH !!!                             #\n");
+    printf("#                                                                   #\n");
+    printf("#  Please help fixing this issue by describing what you were doing  #\n");
+    printf("#    and attaching the above console output in a new ticket at:     #\n");
+    printf("#              http://jackbeat.samalyse.org/newticket               #\n");
+    printf("#                                                                   #\n");
+    printf("#####################################################################\n\n");
 
-  trace_size = backtrace(trace, 200);
-  /* overwrite sigaction with caller's address */
-#ifndef __APPLE__  
-#if defined(__i386__)
-  trace[1] = (void *) uc->uc_mcontext.gregs[REG_EIP];
-#elif defined(__x86_64__)
-  trace[1] = (void *) uc->uc_mcontext.gregs[REG_RIP];
-#endif
-#endif
-
-  backtrace_symbols_fd(trace + 1, trace_size - 1, STDOUT_FILENO);
-
-  printf("#####################################################################\n");
-  printf("#                             KRASH !!!                             #\n");
-  printf("#                                                                   #\n");
-  printf("#  Please help fixing this issue by describing what you were doing  #\n");
-  printf("#    and attaching the above console output in a new ticket at:     #\n");
-  printf("#              http://jackbeat.samalyse.org/newticket               #\n");
-  printf("#                                                                   #\n");
-  printf("#####################################################################\n\n");
-
-  exit(0);
+    exit(0);
 }
+
+void handle_sigsegv_siginfo(int sig, siginfo_t *info, void *secret) {
+    void *trace[200];
+    int trace_size = 0;
+    ucontext_t *uc = (ucontext_t *)secret;
+
+    trace_size = backtrace(trace, 200);
+
+#ifndef __APPLE__
+
+/* overwrite sigaction with caller's address */
+#ifdef __FreeBSD__
+
+#if defined(__i386__)
+    trace[1] = (void *) uc->uc_mcontext.mc_eip;
+#elif defined(__x86_64__)
+    trace[1] = (void *) uc->uc_mcontext.mc_rip;
 #endif
+
+#else /* __FreeBSD__ */
+
+#if defined(__i386__)
+    trace[1] = (void *) uc->uc_mcontext.gregs[REG_EIP];
+#elif defined(__x86_64__)
+    trace[1] = (void *) uc->uc_mcontext.gregs[REG_RIP];
 #endif
 
+#endif /* __FreeBSD__ */
+#endif /* ! __APPLE__ */
+
+    print_backtrace(trace, trace_size);
+}
+
+void handle_sigsegv(int sig) {
+    void *trace[200];
+    int trace_size = 0;
+
+    trace_size = backtrace(trace, 200);
+    print_backtrace(trace, trace_size);
+}
+
+#endif /* HAVE_UCONTEXT_H */
+#endif /* HAVE_EXECINFO_H */
+
 #ifdef HAVE_GTK_QUARTZ
 static pascal OSErr
 osx_desktop_open_handler (const AppleEvent *event, AppleEvent *reply, long refCon)
@@ -135,15 +160,20 @@ main (int argc, char *argv[])
 {
 #ifdef HAVE_EXECINFO_H
 #ifdef HAVE_UCONTEXT_H
-  struct sigaction sa;
+    struct sigaction sa;
 
-  sa.sa_sigaction = (void *)bt_sighandler;
-  sigemptyset (&sa.sa_mask);
-  sa.sa_flags = SA_RESTART | SA_SIGINFO;
+#ifdef SA_SIGINFO
+    sa.sa_sigaction = (void *) handle_sigsegv_siginfo;
+    sa.sa_flags = SA_RESTART | SA_SIGINFO;
+#else
+    sa.sa_sigaction = (void *) handle_sigsegv;
+    sa.sa_flags = SA_RESTART;
+#endif /* SA_SIGINFO  */
 
-  sigaction (SIGSEGV, &sa, NULL);
-#endif
-#endif
+    sigemptyset(&sa.sa_mask);
+    sigaction(SIGSEGV, &sa, NULL);
+#endif /* HAVE_UCONTEXT_H */
+#endif /* HAVE_EXECINFO_H */
 
 #ifdef __WIN32__
   if (AttachConsole(ATTACH_PARENT_PROCESS)) {
