@@ -17,7 +17,7 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   SVN:$Id: sequence.c 610 2009-06-26 12:49:33Z olivier $
+ *   SVN:$Id: sequence.c 643 2009-11-23 19:38:59Z olivier $
  */
 
 #define SEQUENCE_MASK_ATTACK_DELAY 3.0 // Miliseconds
@@ -162,6 +162,9 @@ typedef struct sequence_msg_t
 #define SEQUENCE_MSG_SET_VOLUME     11
 #define SEQUENCE_MSG_MUL_VOLUME     12
 #define SEQUENCE_MSG_SET_SMOOTHING  13
+#define SEQUENCE_MSG_MUTE_TRACK     14
+#define SEQUENCE_MSG_SOLO_TRACK     15
+#define SEQUENCE_MSG_SWAP_TRACKS    16
 
 #define SEQUENCE_MSG_NO_ACK -32
 #define SEQUENCE_MSG_ACK    33
@@ -172,13 +175,11 @@ typedef struct sequence_msg_t
 #define SEQUENCE_MSG_UNLOCK_SINGLE_TRACK  37
 #define SEQUENCE_MSG_RESIZE               38
 #define SEQUENCE_MSG_SET_SAMPLE           39
-#define SEQUENCE_MSG_MUTE_TRACK           40
 #define SEQUENCE_MSG_ENABLE_MASK          41
 #define SEQUENCE_MSG_DISABLE_MASK         42
 #define SEQUENCE_MSG_ACK_STOP             43
 #define SEQUENCE_MSG_ACK_ENABLE           44
 #define SEQUENCE_MSG_ACK_DISABLE          45
-#define SEQUENCE_MSG_SOLO_TRACK           46
 
 
 /*******************************
@@ -544,6 +545,13 @@ sequence_receive_messages (sequence_t * sequence)
         sscanf (msg.text, "track=%d status=%d", &i, &j);
         sequence->tracks[i].smoothing = j;
         break;
+      case SEQUENCE_MSG_SWAP_TRACKS:
+        sscanf (msg.text, "track1=%d track2=%d", &i, &j);
+        sequence_track_t tmp = sequence->tracks[i];
+        sequence->tracks[i]  = sequence->tracks[j];
+        sequence->tracks[j]  = tmp;
+        msg_event_fire (sequence->msg, "reordered", NULL, 0, NULL);
+        break;
     }
   }
 
@@ -829,6 +837,7 @@ sequence_init (sequence_t * sequence)
   event_register (sequence, "track-volume-changed");
   event_register (sequence, "track-name-changed");
   event_register (sequence, "resized");
+  event_register (sequence, "reordered");
   event_register (sequence, "resampler-type-changed");
 }
 
@@ -1616,8 +1625,8 @@ sequence_set_sample (sequence_t * sequence, int track, sample_t * sample)
   return success;
 }
 
-int
-sequence_set_track_name (sequence_t * sequence, int track, char *name)
+static int
+sequence_do_set_track_name (sequence_t * sequence, int track, char *name, int force)
 {
   sequence_lock (sequence);
   int j, error = 0;
@@ -1625,7 +1634,7 @@ sequence_set_track_name (sequence_t * sequence, int track, char *name)
     sequence_track_t *t = sequence->tracks + track;
     if (strcmp (t->name, name))
     {
-      if (!_sequence_track_name_exists (sequence, name)) {
+      if (force || !_sequence_track_name_exists (sequence, name)) {
         if (strlen(name) > 0 && strspn (name, SEQUENCE_VALID_NAME) == strlen (name)) {
           strcpy (t->name, name);
           for (j = 0; j < t->channels_num; j++)
@@ -1649,6 +1658,16 @@ sequence_set_track_name (sequence_t * sequence, int track, char *name)
   return error;
 }
 
+int
+sequence_set_track_name(sequence_t * sequence, int track, char *name) {
+    return sequence_do_set_track_name(sequence, track, name, 0);
+}
+
+int
+sequence_force_track_name(sequence_t * sequence, int track, char *name) {
+    return sequence_do_set_track_name(sequence, track, name, 1);
+}
+
 int
 sequence_get_sample_usage (sequence_t * sequence, sample_t * sample)
 {
@@ -1690,7 +1709,7 @@ sequence_mute_track (sequence_t * sequence, char status, int track)
     sequence_msg_t msg;
     msg.type = SEQUENCE_MSG_MUTE_TRACK;
     sprintf (msg.text, "track=%d mute=%d", track, (status) ? 1 : 0);
-    msg_send (sequence->msg, &msg, MSG_ACK);
+    msg_send (sequence->msg, &msg, 0);
   }
   sequence_unlock (sequence);
 }
@@ -1710,7 +1729,7 @@ sequence_solo_track (sequence_t * sequence, char status, int track)
     sequence_msg_t msg;
     msg.type = SEQUENCE_MSG_SOLO_TRACK;
     sprintf (msg.text, "track=%d solo=%d", track, (status) ? 1 : 0);
-    msg_send (sequence->msg, &msg, MSG_ACK);
+    msg_send (sequence->msg, &msg, 0);
   }
   sequence_unlock (sequence);
 }
@@ -2190,3 +2209,17 @@ sequence_normalize_name (char *name)
 {
     util_strcanon(name, SEQUENCE_VALID_NAME, '_');
 }
+
+void
+sequence_swap_tracks(sequence_t *sequence, int track1, int track2)
+{
+  sequence_lock (sequence);
+  if (sequence_check_pos(sequence, track1, 0) && sequence_check_pos(sequence, track2, 0) 
+          && track1 != track2) {
+      sequence_msg_t msg;
+      msg.type = SEQUENCE_MSG_SWAP_TRACKS;
+      sprintf (msg.text, "track1=%d track2=%d", track1, track2);
+      msg_send (sequence->msg, &msg, 0);
+  }
+  sequence_unlock (sequence);
+}
